@@ -377,6 +377,8 @@ class Hypergraph:
         See Also
         --------
         add_nodes_from
+        set_node_attributes
+        get_node_attributes
 
         Notes
         -----
@@ -406,7 +408,8 @@ class Hypergraph:
         See Also
         --------
         add_node
-
+        set_node_attributes
+        get_node_attributes
         """
         for n in nodes_for_adding:
             try:
@@ -502,6 +505,8 @@ class Hypergraph:
         See Also
         --------
         add_edges_from : Add a collection of edges.
+        set_edge_attributes
+        get_edge_attributes
 
         Examples
         --------
@@ -534,7 +539,7 @@ class Hypergraph:
             warn(f"uid {id} already exists, cannot add edge {members}")
             return
 
-        uid = next(self._edge_uid) if not id else id
+        uid = next(self._edge_uid) if id is None else id
 
         self._edge[uid] = set()
         for node in members:
@@ -583,6 +588,8 @@ class Hypergraph:
         --------
         add_edge : Add a single edge.
         add_weighted_edges_from : Convenient way to add weighted edges.
+        set_edge_attributes
+        get_edge_attributes
 
         Notes
         -----
@@ -714,7 +721,6 @@ class Hypergraph:
             if id in self._edge.keys():  # check that uid is not present yet
                 warn(f"uid {id} already exists, cannot add edge {members}.")
             else:
-
                 try:
                     self._edge[id] = set(members)
                 except TypeError as e:
@@ -733,7 +739,6 @@ class Hypergraph:
             try:
                 e = next(new_edges)
             except StopIteration:
-
                 if format2 or format4:
                     update_uid_counter(self, id)
                 break
@@ -756,6 +761,8 @@ class Hypergraph:
         --------
         add_edge : Add a single edge.
         add_edges_from : Add multiple edges.
+        set_edge_attributes
+        get_edge_attributes
 
         Notes
         -----
@@ -811,7 +818,6 @@ class Hypergraph:
         """
         # Assign edges to modify
         try:
-
             temp_memberships1 = list(self._node[n_id1])
             temp_memberships1[temp_memberships1.index(e_id1)] = e_id2
 
