@@ -1,5 +1,6 @@
 from collections import defaultdict
 from copy import deepcopy
+from itertools import combinations
 
 import networkx as nx
 import pandas as pd
@@ -38,11 +39,12 @@ __all__ = [
     "from_bipartite_pandas_dataframe",
     "to_bipartite_pandas_dataframe",
     "from_incidence_matrix",
-    "from_simplicial_complex_to_hypergraph",
+    "from_max_simplices",
     "to_incidence_matrix",
     "from_bipartite_graph",
     "to_bipartite_graph",
     "dict_to_hypergraph",
+    "to_line_graph",
 ]
 
 
@@ -134,6 +136,46 @@ def convert_to_graph(H):
     return G
 
 
+def to_line_graph(H, s=1):
+    """The s-line graph of the hypergraph.
+
+    The line graph of the hypergraph `H` is the graph whose
+    nodes correspond to each hyperedge in `H`, linked together
+    if they share at least one vertex.
+
+    Parameters
+    ----------
+    H : Hypergraph
+        The hypergraph of interest
+    s : int
+        The intersection size to consider edges
+        as connected, by default 1.
+
+    Returns
+    -------
+    LG : networkx.Graph
+         The line graph associated to the Hypergraph
+
+    References
+    ----------
+    "Hypernetwork science via high-order hypergraph walks"
+    by Sinan G. Aksoy, Cliff Joslyn, Carlos Ortiz Marrero, Brenda Praggastis & Emilie Purvine.
+    https://doi.org/10.1140/epjds/s13688-020-00231-0
+    """
+
+    LG = nx.Graph()
+
+    edge_label_dict = {tuple(edge): index for index, edge in H._edge.items()}
+
+    LG.add_nodes_from(H.edges)
+
+    for edge1, edge2 in combinations(H.edges.members(), 2):
+        if len(edge1.intersection(edge2)) >= s:
+            LG.add_edge(edge_label_dict[tuple(edge1)], edge_label_dict[tuple(edge2)])
+
+    return LG
+
+
 def convert_to_simplicial_complex(data, create_using=None):
     """Make a hypergraph from a known data structure.
     The preferred way to call this is automatically
@@ -442,9 +484,9 @@ def from_incidence_matrix(d, create_using=None, nodelabels=None, edgelabels=None
     return H
 
 
-def from_simplicial_complex_to_hypergraph(SC):
+def from_max_simplices(SC):
     """Returns a hypergraph constructed from the
-    maximal simpices of the provided simplicial complex.
+    maximal simplices of the provided simplicial complex.
 
     Parameters
     ----------
@@ -460,6 +502,7 @@ def from_simplicial_complex_to_hypergraph(SC):
 
     max_simplices = maximal_simplices(SC)
     H = Hypergraph()
+    H.add_nodes_from(SC.nodes)  # to keep node order and isolated nodes
     H.add_edges_from([list(SC.edges.members(e)) for e in max_simplices])
     return H
 
