@@ -1,9 +1,55 @@
-"""Matrices associated to hypergraphs."""
-from warnings import warn
+"""Matrices associated to hypergraphs.
+
+Note that the order of the rows and columns of the
+matrices in this module correspond to the order in
+which nodes/edges are added to the hypergraph or
+simplicial complex. If the node and edge IDs are
+able to be sorted, the following is an example to sort
+by the node and edge IDs.
+
+>>> import xgi
+>>> import pandas as pd
+>>> H = xgi.Hypergraph([[1, 2, 3, 7], [4], [5, 6, 7]])
+>>> I, nodedict, edgedict = xgi.incidence_matrix(H, sparse=False, index=True)
+>>> # Sorting the resulting numpy array:
+>>> sortedI = I.copy()
+>>> sortedI = sortedI[sorted(nodedict, key=nodedict.get), :]
+>>> sortedI = sortedI[:, sorted(edgedict, key=edgedict.get)]
+>>> sortedI
+array([[1, 0, 0],
+       [1, 0, 0],
+       [1, 0, 0],
+       [0, 1, 0],
+       [0, 0, 1],
+       [0, 0, 1],
+       [1, 0, 1]])
+>>> # Indexing a Pandas dataframe by the node/edge IDs
+>>> df = pd.DataFrame(I, index=nodedict.values(), columns=edgedict.values())
+
+If the nodes are already sorted, this order can be preserved by adding the nodes
+to the hypergraph prior to adding edges. For example,
+
+>>> import xgi
+>>> H = xgi.Hypergraph()
+>>> H.add_nodes_from(range(1, 8))
+>>> H.add_edges_from([[1, 2, 3, 7], [4], [5, 6, 7]])
+>>> xgi.incidence_matrix(H, sparse=False)
+array([[1, 0, 0],
+       [1, 0, 0],
+       [1, 0, 0],
+       [0, 1, 0],
+       [0, 0, 1],
+       [0, 0, 1],
+       [1, 0, 1]])
+
+"""
+from warnings import catch_warnings, warn
 
 import numpy as np
 from scipy.sparse import csr_array, diags
 
+from ..exception import XGIError
+
 __all__ = [
     "incidence_matrix",
     "adjacency_matrix",
@@ -11,6 +57,7 @@ __all__ = [
     "degree_matrix",
     "laplacian",
     "multiorder_laplacian",
+    "normalized_hypergraph_laplacian",
     "clique_motif_matrix",
     "boundary_matrix",
     "hodge_laplacian",
@@ -54,7 +101,11 @@ def incidence_matrix(
     if order is not None:
         edge_ids = H.edges.filterby("order", order)
     if not edge_ids or not node_ids:
-        return (np.array([]), {}, {}) if index else np.array([])
+        if sparse:
+            I = csr_array((0, 0), dtype=int)
+        else:
+            I = np.empty((0, 0), dtype=int)
+        return (I, {}, {}) if index else I
 
     num_edges = len(edge_ids)
     num_nodes = len(node_ids)
@@ -66,40 +117,28 @@ def incidence_matrix(
         rowdict = {v: k for k, v in node_dict.items()}
         coldict = {v: k for k, v in edge_dict.items()}
 
+    # Compute the non-zero values, row and column indices for the given order
+    rows = []
+    cols = []
+    data = []
+    for edge in edge_ids:
+        members = H._edge[edge]
+        for node in members:
+            rows.append(node_dict[node])
+            cols.append(edge_dict[edge])
+            data.append(weight(node, edge, H))
+
+    # Create the incidence matrix as a CSR matrix
     if sparse:
-        # Create csr sparse matrix
-        rows = []
-        cols = []
-        data = []
-        for node in node_ids:
-            memberships = H.nodes.memberships(node)
-            # keep only those with right order
-            memberships = [i for i in memberships if i in edge_ids]
-            if len(memberships) > 0:
-                for edge in memberships:
-                    data.append(weight(node, edge, H))
-                    rows.append(node_dict[node])
-                    cols.append(edge_dict[edge])
-            else:  # include disconnected nodes
-                for edge in edge_ids:
-                    data.append(0)
-                    rows.append(node_dict[node])
-                    cols.append(edge_dict[edge])
-        I = csr_array((data, (rows, cols)))
+        I = csr_array((data, (rows, cols)), shape=(num_nodes, num_edges), dtype=int)
     else:
-        # Create an np.matrix
         I = np.zeros((num_nodes, num_edges), dtype=int)
-        for edge in edge_ids:
-            members = H.edges.members(edge)
-            for node in members:
-                I[node_dict[node], edge_dict[edge]] = weight(node, edge, H)
-    if index:
-        return I, rowdict, coldict
-    else:
-        return I
+        I[rows, cols] = data
+
+    return (I, rowdict, coldict) if index else I
 
 
-def adjacency_matrix(H, order=None, s=1, weighted=False, index=False):
+def adjacency_matrix(H, order=None, sparse=True, s=1, weighted=False, index=False):
     """
     A function to generate an adjacency matrix (N,N) from a Hypergraph object.
 
@@ -110,6 +149,8 @@ def adjacency_matrix(H, order=None, s=1, weighted=False, index=False):
     order: int, optional
         Order of interactions to use. If None (default), all orders are used. If int,
         must be >= 1.
+    sparse: bool, default: True
+        Specifies whether the output matrix is a scipy sparse matrix or a numpy matrix
     s: int, default: 1
         Specifies the number of overlapping edges to be considered connected.
     index: bool, default: False
@@ -122,31 +163,46 @@ def adjacency_matrix(H, order=None, s=1, weighted=False, index=False):
     else:
         return A
 
-    """
-    I, rowdict, coldict = incidence_matrix(H, index=True, order=order)
+    Warns
+    -----
+    warn
+        If there are isolated nodes and the matrix is sparse.
 
-    if I.shape == (0,):
+    """
+    I, rowdict, coldict = incidence_matrix(H, order=order, sparse=sparse, index=True)
+
+    if I.shape == (0, 0):
         if not rowdict:
-            A = np.array([])
+            A = csr_array((0, 0)) if sparse else np.empty((0, 0))
         if not coldict:
-            A = np.zeros((H.num_nodes, H.num_nodes))
+            shape = (H.num_nodes, H.num_nodes)
+            A = csr_array(shape, dtype=int) if sparse else np.zeros(shape, dtype=int)
         return (A, {}) if index else A
 
     A = I.dot(I.T)
-    A = A - diags(A.diagonal())
+
+    if sparse:
+        with catch_warnings(record=True) as w:
+            A.setdiag(0)
+        if w:
+            warn(
+                "Forming the adjacency matrix can be expensive when there are isolated nodes!"
+            )
+    else:
+        np.fill_diagonal(A, 0)
 
     if not weighted:
         A = (A >= s) * 1
     else:
-        A.data[np.where(A.data < s)] = 0
+        A = (A >= s) * A
 
-    if index:
-        return A, rowdict
-    else:
-        return A
+    if sparse:
+        A.eliminate_zeros()
+
+    return (A, rowdict) if index else A
 
 
-def intersection_profile(H, order=None, index=False):
+def intersection_profile(H, order=None, sparse=True, index=False):
     """
     A function to generate an intersection profile from a Hypergraph object.
 
@@ -157,6 +213,8 @@ def intersection_profile(H, order=None, index=False):
     order: int, optional
         Order of interactions to use. If None (default), all orders are used. If int,
         must be >= 1.
+    sparse: bool, default: True
+        Specifies whether the output matrix is a scipy sparse matrix or a numpy matrix
     index: bool, default: False
         Specifies whether to output dictionaries mapping the edge IDs to indices
 
@@ -168,18 +226,9 @@ def intersection_profile(H, order=None, index=False):
         return P
 
     """
-
-    if index:
-        I, _, coldict = incidence_matrix(H, order=order, index=True)
-    else:
-        I = incidence_matrix(H, order=order, index=False)
-
+    I, _, coldict = incidence_matrix(H, order=order, sparse=sparse, index=True)
     P = I.T.dot(I)
-
-    if index:
-        return P, coldict
-    else:
-        return P
+    return (P, coldict) if index else P
 
 
 def degree_matrix(H, order=None, index=False):
@@ -205,7 +254,7 @@ def degree_matrix(H, order=None, index=False):
     """
     I, rowdict, _ = incidence_matrix(H, order=order, index=True)
 
-    if I.shape == (0,):
+    if I.shape == (0, 0):
         K = np.zeros(H.num_nodes)
     else:
         K = np.ravel(np.sum(I, axis=1))  # flatten
@@ -213,7 +262,7 @@ def degree_matrix(H, order=None, index=False):
     return (K, rowdict) if index else K
 
 
-def laplacian(H, order=1, rescale_per_node=False, index=False):
+def laplacian(H, order=1, sparse=False, rescale_per_node=False, index=False):
     """Laplacian matrix of order d, see [1].
 
     Parameters
@@ -223,6 +272,8 @@ def laplacian(H, order=1, rescale_per_node=False, index=False):
     order : int
         Order of interactions to consider. If order=1 (default),
         returns the usual graph Laplacian
+    sparse: bool, default: False
+        Specifies whether the output matrix is a scipy sparse matrix or a numpy matrix
     index: bool, default: False
         Specifies whether to output disctionaries mapping the node and edge IDs to indices
 
@@ -244,25 +295,30 @@ def laplacian(H, order=1, rescale_per_node=False, index=False):
         Physical Review Research, 2(3), 033410.
 
     """
-    A, row_dict = adjacency_matrix(H, order=order, weighted=True, index=True)
-    if A.shape == (0,):
-        return (np.array([]), {}) if index else np.array([])
+    A, row_dict = adjacency_matrix(
+        H, order=order, sparse=sparse, weighted=True, index=True
+    )
 
-    K = degree_matrix(H, order=order, index=False)
+    if A.shape == (0, 0):
+        L = csr_array((0, 0)) if sparse else np.empty((0, 0))
+        return (L, {}) if index else L
 
-    L = order * np.diag(K) - A  # ravel needed to convert sparse matrix
-    L = np.asarray(L)
+    if sparse:
+        K = csr_array(diags(degree_matrix(H, order=order)))
+    else:
+        K = np.diag(degree_matrix(H, order=order))
+
+    L = order * K - A  # ravel needed to convert sparse matrix
 
     if rescale_per_node:
         L = L / order
 
-    if index:
-        return L, row_dict
-    else:
-        return L
+    return (L, row_dict) if index else L
 
 
-def multiorder_laplacian(H, orders, weights, rescale_per_node=False, index=False):
+def multiorder_laplacian(
+    H, orders, weights, sparse=False, rescale_per_node=False, index=False
+):
     """Multiorder Laplacian matrix, see [1].
 
     Parameters
@@ -273,6 +329,8 @@ def multiorder_laplacian(H, orders, weights, rescale_per_node=False, index=False
         Orders of interactions to consider.
     weights: list of float
         Weights associated to each order, i.e coupling strengths gamma_i in [1].
+    sparse: bool, default: False
+        Specifies whether the output matrix is a scipy sparse matrix or a numpy matrix
     rescale_per_node: bool, (default=False)
         Whether to rescale each Laplacian of order d by d (per node).
     index: bool, default: False
@@ -299,10 +357,17 @@ def multiorder_laplacian(H, orders, weights, rescale_per_node=False, index=False
     if len(orders) != len(weights):
         raise ValueError("orders and weights must have the same length.")
 
-    Ls = [laplacian(H, order=i, rescale_per_node=rescale_per_node) for i in orders]
-    Ks = [degree_matrix(H, order=i) for i in orders]
+    Ls = [
+        laplacian(H, order=d, sparse=sparse, rescale_per_node=rescale_per_node)
+        for d in orders
+    ]
+    Ks = [degree_matrix(H, order=d) for d in orders]
+
+    if sparse:
+        L_multi = csr_array((H.num_nodes, H.num_nodes))
+    else:
+        L_multi = np.zeros((H.num_nodes, H.num_nodes))
 
-    L_multi = np.zeros((H.num_nodes, H.num_nodes))
     for L, K, w, d in zip(Ls, Ks, weights, orders):
         if np.all(K == 0):
             # avoid getting nans from dividing by 0
@@ -313,14 +378,67 @@ def multiorder_laplacian(H, orders, weights, rescale_per_node=False, index=False
         else:
             L_multi += L * w / np.mean(K)
 
-    if index:
-        _, rowdict, _ = incidence_matrix(H, index=True)
-        return L_multi, rowdict
+    rowdict = {i: v for i, v in enumerate(H.nodes)}
+
+    return (L_multi, rowdict) if index else L_multi
+
+
+def normalized_hypergraph_laplacian(H, sparse=True, index=False):
+    """Compute the normalized Laplacian.
+
+    Parameters
+    ----------
+    H : Hypergraph
+        Hypergraph
+    sparse : bool, optional
+        whether or not the laplacian is sparse, by default True
+    index : bool, optional
+        whether to return a dictionary mapping IDs to rows, by default False
+
+    Returns
+    -------
+    array
+        csr_array if sparse and if not, a numpy ndarray
+    dict
+        a dictionary mapping node IDs to rows and columns
+        if index is True.
+
+
+    Raises
+    ------
+    XGIError
+        If there are isolated nodes.
+
+    References
+    ----------
+    "Learning with Hypergraphs: Clustering, Classification, and Embedding"
+    by Dengyong Zhou, Jiayuan Huang, Bernhard Schölkopf
+    Advances in Neural Information Processing Systems (2006)
+    """
+
+    from ..algorithms import is_connected
+
+    if H.nodes.isolates():
+        raise XGIError(
+            "Every node must be a member of an edge to avoid divide by zero error!"
+        )
+
+    D = degree_matrix(H)
+    A, rowdict = clique_motif_matrix(H, sparse=sparse, index=True)
+
+    if sparse:
+        Dinvsqrt = csr_array(diags(np.power(D, -0.5)))
+        I = csr_array((H.num_nodes, H.num_nodes))
+        I.setdiag(1)
     else:
-        return L_multi
+        Dinvsqrt = np.diag(np.power(D, -0.5))
+        I = np.eye(H.num_nodes)
+
+    L = 0.5 * (I - Dinvsqrt @ A @ Dinvsqrt)
+    return (L, rowdict) if index else L
 
 
-def clique_motif_matrix(H, index=False):
+def clique_motif_matrix(H, sparse=True, index=False):
     """
     A function to generate a weighted clique motif matrix
     from a Hypergraph object.
@@ -329,6 +447,8 @@ def clique_motif_matrix(H, index=False):
     ----------
     H: Hypergraph object
         The hypergraph of interest
+    sparse: bool, default: True
+        Specifies whether the output matrix is a scipy sparse matrix or a numpy matrix
     index: bool, default: False
         Specifies whether to output dictionaries
         mapping the node and edge IDs to indices
@@ -336,7 +456,7 @@ def clique_motif_matrix(H, index=False):
     Returns
     -------
     if index is True:
-        return W, rowdict, coldict
+        return W, rowdict
     else:
         return W
 
@@ -347,22 +467,9 @@ def clique_motif_matrix(H, index=False):
     https://doi.org/10.1126/science.aad9029
 
     """
-    if index:
-        I, rowdict, _ = incidence_matrix(H, index=True)
-    else:
-        I = incidence_matrix(H, index=False)
+    W, rowdict = adjacency_matrix(H, sparse=sparse, weighted=True, index=True)
 
-    if I.shape == (0,):
-        return (np.array([]), rowdict) if index else np.array([])
-
-    W = I.dot(I.T)
-    W.setdiag(0)
-    W.eliminate_zeros()
-
-    if index:
-        return W, rowdict
-    else:
-        return W
+    return (W, rowdict) if index else W
 
 
 def boundary_matrix(S, order=1, orientations=None, index=False):
@@ -467,14 +574,10 @@ def boundary_matrix(S, order=1, orientations=None, index=False):
                     B[simplices_d_dict[subface_ID], matrix_id] = (-1) ** (
                         subfaces_induced_orientation[count] + orientations[subface_ID]
                     )
-    if index:
-        return B, rowdict, coldict
-    else:
-        return B
+    return (B, rowdict, coldict) if index else B
 
 
 def hodge_laplacian(S, order=1, orientations=None, index=False):
-
     """
     A function to compute the Hodge Laplacians of an oriented
     simplicial complex.
@@ -514,7 +617,4 @@ def hodge_laplacian(S, order=1, orientations=None, index=False):
 
     L_o = D_om1 @ B_o + B_op1 @ D_o
 
-    if index:
-        return L_o, matdict
-    else:
-        return L_o
+    return (L_o, matdict) if index else L_o
